# Generated from agenterprise/agent_grammer/parser/ai_environment.g4 by ANTLR 4.13.2
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,55,341,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,2,4,7,4,2,5,7,5,2,6,7,
        6,2,7,7,7,2,8,7,8,2,9,7,9,2,10,7,10,2,11,7,11,2,12,7,12,2,13,7,13,
        2,14,7,14,2,15,7,15,2,16,7,16,2,17,7,17,2,18,7,18,2,19,7,19,2,20,
        7,20,2,21,7,21,2,22,7,22,2,23,7,23,2,24,7,24,2,25,7,25,2,26,7,26,
        2,27,7,27,2,28,7,28,2,29,7,29,2,30,7,30,2,31,7,31,2,32,7,32,2,33,
        7,33,2,34,7,34,2,35,7,35,2,36,7,36,2,37,7,37,2,38,7,38,2,39,7,39,
        2,40,7,40,2,41,7,41,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,
        1,0,1,0,1,0,5,0,99,8,0,10,0,12,0,102,9,0,1,0,1,0,1,0,1,0,5,0,108,
        8,0,10,0,12,0,111,9,0,1,0,1,0,1,0,1,0,5,0,117,8,0,10,0,12,0,120,
        9,0,1,0,5,0,123,8,0,10,0,12,0,126,9,0,1,0,1,0,1,0,1,1,1,1,1,1,1,
        1,1,2,1,2,1,2,1,2,1,3,1,3,1,3,1,3,1,4,1,4,1,4,1,4,1,5,1,5,1,5,1,
        5,1,6,1,6,1,6,1,6,5,6,155,8,6,10,6,12,6,158,9,6,1,6,1,6,1,7,1,7,
        3,7,164,8,7,1,8,1,8,1,8,1,8,1,9,1,9,1,9,1,9,1,9,1,9,1,10,1,10,1,
        10,1,11,1,11,1,11,1,12,1,12,1,12,1,12,5,12,186,8,12,10,12,12,12,
        189,9,12,1,12,1,12,1,13,1,13,1,13,1,13,1,13,1,13,1,13,1,13,1,13,
        1,13,1,13,3,13,204,8,13,1,14,1,14,1,14,1,14,1,15,1,15,1,15,1,15,
        1,16,1,16,1,16,1,16,1,17,1,17,1,17,1,17,1,18,1,18,1,18,1,18,1,19,
        1,19,1,19,1,19,1,20,1,20,1,20,1,20,1,21,1,21,1,21,1,21,1,22,1,22,
        1,22,1,22,1,23,1,23,1,23,1,23,1,24,1,24,1,24,1,24,1,25,1,25,1,25,
        1,25,5,25,254,8,25,10,25,12,25,257,9,25,1,25,1,25,1,26,1,26,1,26,
        1,26,1,26,1,26,3,26,267,8,26,1,27,1,27,1,27,1,27,1,28,1,28,1,28,
        1,28,1,29,1,29,1,29,1,29,1,30,1,30,1,30,1,30,1,31,1,31,1,31,1,31,
        1,32,1,32,1,32,1,32,1,33,1,33,1,33,1,33,5,33,297,8,33,10,33,12,33,
        300,9,33,1,33,1,33,1,34,1,34,1,34,1,34,1,34,1,34,1,34,3,34,311,8,
        34,1,35,1,35,1,35,1,35,1,36,1,36,1,36,1,36,1,37,1,37,1,37,1,37,1,
        38,1,38,1,38,1,38,1,39,1,39,1,39,1,39,1,40,1,40,1,40,1,40,1,41,1,
        41,1,41,1,41,1,41,0,0,42,0,2,4,6,8,10,12,14,16,18,20,22,24,26,28,
        30,32,34,36,38,40,42,44,46,48,50,52,54,56,58,60,62,64,66,68,70,72,
        74,76,78,80,82,0,2,1,0,40,41,2,0,39,39,42,42,328,0,84,1,0,0,0,2,
        130,1,0,0,0,4,134,1,0,0,0,6,138,1,0,0,0,8,142,1,0,0,0,10,146,1,0,
        0,0,12,150,1,0,0,0,14,163,1,0,0,0,16,165,1,0,0,0,18,169,1,0,0,0,
        20,175,1,0,0,0,22,178,1,0,0,0,24,181,1,0,0,0,26,203,1,0,0,0,28,205,
        1,0,0,0,30,209,1,0,0,0,32,213,1,0,0,0,34,217,1,0,0,0,36,221,1,0,
        0,0,38,225,1,0,0,0,40,229,1,0,0,0,42,233,1,0,0,0,44,237,1,0,0,0,
        46,241,1,0,0,0,48,245,1,0,0,0,50,249,1,0,0,0,52,266,1,0,0,0,54,268,
        1,0,0,0,56,272,1,0,0,0,58,276,1,0,0,0,60,280,1,0,0,0,62,284,1,0,
        0,0,64,288,1,0,0,0,66,292,1,0,0,0,68,310,1,0,0,0,70,312,1,0,0,0,
        72,316,1,0,0,0,74,320,1,0,0,0,76,324,1,0,0,0,78,328,1,0,0,0,80,332,
        1,0,0,0,82,336,1,0,0,0,84,85,5,1,0,0,85,86,5,51,0,0,86,87,5,2,0,
        0,87,88,5,3,0,0,88,89,5,2,0,0,89,90,3,2,1,0,90,91,3,4,2,0,91,92,
        3,6,3,0,92,93,3,8,4,0,93,94,3,10,5,0,94,95,5,4,0,0,95,96,5,5,0,0,
        96,100,5,2,0,0,97,99,3,12,6,0,98,97,1,0,0,0,99,102,1,0,0,0,100,98,
        1,0,0,0,100,101,1,0,0,0,101,103,1,0,0,0,102,100,1,0,0,0,103,104,
        5,4,0,0,104,105,5,6,0,0,105,109,5,2,0,0,106,108,3,50,25,0,107,106,
        1,0,0,0,108,111,1,0,0,0,109,107,1,0,0,0,109,110,1,0,0,0,110,112,
        1,0,0,0,111,109,1,0,0,0,112,113,5,4,0,0,113,114,5,7,0,0,114,118,
        5,2,0,0,115,117,3,24,12,0,116,115,1,0,0,0,117,120,1,0,0,0,118,116,
        1,0,0,0,118,119,1,0,0,0,119,124,1,0,0,0,120,118,1,0,0,0,121,123,
        3,66,33,0,122,121,1,0,0,0,123,126,1,0,0,0,124,122,1,0,0,0,124,125,
        1,0,0,0,125,127,1,0,0,0,126,124,1,0,0,0,127,128,5,4,0,0,128,129,
        5,4,0,0,129,1,1,0,0,0,130,131,5,8,0,0,131,132,5,9,0,0,132,133,5,
        51,0,0,133,3,1,0,0,0,134,135,5,10,0,0,135,136,5,9,0,0,136,137,5,
        37,0,0,137,5,1,0,0,0,138,139,5,11,0,0,139,140,5,9,0,0,140,141,5,
        37,0,0,141,7,1,0,0,0,142,143,5,12,0,0,143,144,5,9,0,0,144,145,5,
        37,0,0,145,9,1,0,0,0,146,147,5,13,0,0,147,148,5,9,0,0,148,149,5,
        37,0,0,149,11,1,0,0,0,150,151,5,14,0,0,151,152,5,51,0,0,152,156,
        5,2,0,0,153,155,3,14,7,0,154,153,1,0,0,0,155,158,1,0,0,0,156,154,
        1,0,0,0,156,157,1,0,0,0,157,159,1,0,0,0,158,156,1,0,0,0,159,160,
        5,4,0,0,160,13,1,0,0,0,161,164,3,16,8,0,162,164,3,18,9,0,163,161,
        1,0,0,0,163,162,1,0,0,0,164,15,1,0,0,0,165,166,5,15,0,0,166,167,
        5,9,0,0,167,168,5,39,0,0,168,17,1,0,0,0,169,170,5,16,0,0,170,171,
        5,9,0,0,171,172,7,0,0,0,172,173,3,20,10,0,173,174,3,22,11,0,174,
        19,1,0,0,0,175,176,5,17,0,0,176,177,7,1,0,0,177,21,1,0,0,0,178,179,
        5,18,0,0,179,180,5,51,0,0,180,23,1,0,0,0,181,182,5,19,0,0,182,183,
        5,51,0,0,183,187,5,2,0,0,184,186,3,26,13,0,185,184,1,0,0,0,186,189,
        1,0,0,0,187,185,1,0,0,0,187,188,1,0,0,0,188,190,1,0,0,0,189,187,
        1,0,0,0,190,191,5,4,0,0,191,25,1,0,0,0,192,204,3,28,14,0,193,204,
        3,30,15,0,194,204,3,32,16,0,195,204,3,34,17,0,196,204,3,36,18,0,
        197,204,3,38,19,0,198,204,3,40,20,0,199,204,3,42,21,0,200,204,3,
        44,22,0,201,204,3,46,23,0,202,204,3,48,24,0,203,192,1,0,0,0,203,
        193,1,0,0,0,203,194,1,0,0,0,203,195,1,0,0,0,203,196,1,0,0,0,203,
        197,1,0,0,0,203,198,1,0,0,0,203,199,1,0,0,0,203,200,1,0,0,0,203,
        201,1,0,0,0,203,202,1,0,0,0,204,27,1,0,0,0,205,206,5,20,0,0,206,
        207,5,9,0,0,207,208,5,51,0,0,208,29,1,0,0,0,209,210,5,15,0,0,210,
        211,5,9,0,0,211,212,5,48,0,0,212,31,1,0,0,0,213,214,5,21,0,0,214,
        215,5,9,0,0,215,216,5,49,0,0,216,33,1,0,0,0,217,218,5,22,0,0,218,
        219,5,9,0,0,219,220,5,44,0,0,220,35,1,0,0,0,221,222,5,23,0,0,222,
        223,5,9,0,0,223,224,5,46,0,0,224,37,1,0,0,0,225,226,5,24,0,0,226,
        227,5,9,0,0,227,228,5,39,0,0,228,39,1,0,0,0,229,230,5,25,0,0,230,
        231,5,9,0,0,231,232,5,39,0,0,232,41,1,0,0,0,233,234,5,26,0,0,234,
        235,5,9,0,0,235,236,5,51,0,0,236,43,1,0,0,0,237,238,5,27,0,0,238,
        239,5,9,0,0,239,240,5,51,0,0,240,45,1,0,0,0,241,242,5,28,0,0,242,
        243,5,9,0,0,243,244,5,51,0,0,244,47,1,0,0,0,245,246,5,38,0,0,246,
        247,5,9,0,0,247,248,5,51,0,0,248,49,1,0,0,0,249,250,5,29,0,0,250,
        251,5,51,0,0,251,255,5,2,0,0,252,254,3,52,26,0,253,252,1,0,0,0,254,
        257,1,0,0,0,255,253,1,0,0,0,255,256,1,0,0,0,256,258,1,0,0,0,257,
        255,1,0,0,0,258,259,5,4,0,0,259,51,1,0,0,0,260,267,3,54,27,0,261,
        267,3,56,28,0,262,267,3,58,29,0,263,267,3,60,30,0,264,267,3,62,31,
        0,265,267,3,64,32,0,266,260,1,0,0,0,266,261,1,0,0,0,266,262,1,0,
        0,0,266,263,1,0,0,0,266,264,1,0,0,0,266,265,1,0,0,0,267,53,1,0,0,
        0,268,269,5,15,0,0,269,270,5,9,0,0,270,271,5,44,0,0,271,55,1,0,0,
        0,272,273,5,30,0,0,273,274,5,9,0,0,274,275,5,43,0,0,275,57,1,0,0,
        0,276,277,5,31,0,0,277,278,5,9,0,0,278,279,5,51,0,0,279,59,1,0,0,
        0,280,281,5,32,0,0,281,282,5,9,0,0,282,283,5,51,0,0,283,61,1,0,0,
        0,284,285,5,33,0,0,285,286,5,9,0,0,286,287,5,51,0,0,287,63,1,0,0,
        0,288,289,5,38,0,0,289,290,5,9,0,0,290,291,5,51,0,0,291,65,1,0,0,
        0,292,293,5,34,0,0,293,294,5,51,0,0,294,298,5,2,0,0,295,297,3,68,
        34,0,296,295,1,0,0,0,297,300,1,0,0,0,298,296,1,0,0,0,298,299,1,0,
        0,0,299,301,1,0,0,0,300,298,1,0,0,0,301,302,5,4,0,0,302,67,1,0,0,
        0,303,311,3,70,35,0,304,311,3,74,37,0,305,311,3,76,38,0,306,311,
        3,72,36,0,307,311,3,80,40,0,308,311,3,78,39,0,309,311,3,82,41,0,
        310,303,1,0,0,0,310,304,1,0,0,0,310,305,1,0,0,0,310,306,1,0,0,0,
        310,307,1,0,0,0,310,308,1,0,0,0,310,309,1,0,0,0,311,69,1,0,0,0,312,
        313,5,15,0,0,313,314,5,9,0,0,314,315,5,46,0,0,315,71,1,0,0,0,316,
        317,5,32,0,0,317,318,5,9,0,0,318,319,5,51,0,0,319,73,1,0,0,0,320,
        321,5,24,0,0,321,322,5,9,0,0,322,323,5,39,0,0,323,75,1,0,0,0,324,
        325,5,25,0,0,325,326,5,9,0,0,326,327,5,39,0,0,327,77,1,0,0,0,328,
        329,5,26,0,0,329,330,5,9,0,0,330,331,5,51,0,0,331,79,1,0,0,0,332,
        333,5,35,0,0,333,334,5,9,0,0,334,335,5,47,0,0,335,81,1,0,0,0,336,
        337,5,38,0,0,337,338,5,9,0,0,338,339,5,51,0,0,339,83,1,0,0,0,12,
        100,109,118,124,156,163,187,203,255,266,298,310
    ]

class ai_environmentParser ( Parser ):

    grammarFileName = "ai_environment.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'ai_environment'", "'{'", "'architecture'", 
                     "'}'", "'data'", "'infrastructure'", "'functional'", 
                     "'envid'", "'='", "'service-techlayer'", "'ai-techlayer'", 
                     "'data-techlayer'", "'agentic-middleware-techlayer'", 
                     "'entity'", "'uid'", "'element'", "'->'", "'#'", "'agent'", 
                     "'systemprompt'", "'namespace'", "'llmref'", "'toolref'", 
                     "'in'", "'out'", "'description'", "'example'", "'tag'", 
                     "'llm'", "'provider'", "'model'", "'endpoint'", "'version'", 
                     "'tool'", "'type'" ]

    symbolicNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                      "TECHLAYER_RESSOURCE", "TECHLAYER_AIURN", "VAR", "ENTITY_ID", 
                      "ENTITY_VAR", "ENTITY_CONSTANT", "ENTITY_TYPE", "LLMPROVIDER", 
                      "LLMID", "TOOLVAR", "TOOLID", "TOOL_TYPE", "AGENTID", 
                      "AGENTNAMESPACE", "AGENTVAR", "PROPERTYVALUE", "THINK", 
                      "WS", "COMMENT", "ML_COMMENT" ]

    RULE_ai_envDef = 0
    RULE_envId = 1
    RULE_architectureServiceStack = 2
    RULE_architectureAiStack = 3
    RULE_architectureDataStack = 4
    RULE_architectureAgenticMiddlewareStack = 5
    RULE_entityDef = 6
    RULE_entityProp = 7
    RULE_entityIdProp = 8
    RULE_entityElementProp = 9
    RULE_entityElementType = 10
    RULE_entityElementDescription = 11
    RULE_agentDef = 12
    RULE_agentProperty = 13
    RULE_agentSystemPromptProperty = 14
    RULE_agentIdentity = 15
    RULE_agentNamespace = 16
    RULE_agentLLMRefProperty = 17
    RULE_agentToolRefProperty = 18
    RULE_agentInputProperty = 19
    RULE_agentOutputProperty = 20
    RULE_agentDescriptionProp = 21
    RULE_agentExampleProp = 22
    RULE_agentTagsProp = 23
    RULE_agentCustomProperty = 24
    RULE_llmDef = 25
    RULE_llmProp = 26
    RULE_llmIdProp = 27
    RULE_llmProviderProp = 28
    RULE_llmModelProp = 29
    RULE_llmEndpointProp = 30
    RULE_llmVersionProp = 31
    RULE_llmOtherProperty = 32
    RULE_toolDef = 33
    RULE_toolProp = 34
    RULE_toolIdProp = 35
    RULE_toolEndpointProp = 36
    RULE_toolInputProperty = 37
    RULE_toolOutputProperty = 38
    RULE_toolDescriptionProp = 39
    RULE_toolTypeProp = 40
    RULE_toolOtherProperty = 41

    ruleNames =  [ "ai_envDef", "envId", "architectureServiceStack", "architectureAiStack", 
                   "architectureDataStack", "architectureAgenticMiddlewareStack", 
                   "entityDef", "entityProp", "entityIdProp", "entityElementProp", 
                   "entityElementType", "entityElementDescription", "agentDef", 
                   "agentProperty", "agentSystemPromptProperty", "agentIdentity", 
                   "agentNamespace", "agentLLMRefProperty", "agentToolRefProperty", 
                   "agentInputProperty", "agentOutputProperty", "agentDescriptionProp", 
                   "agentExampleProp", "agentTagsProp", "agentCustomProperty", 
                   "llmDef", "llmProp", "llmIdProp", "llmProviderProp", 
                   "llmModelProp", "llmEndpointProp", "llmVersionProp", 
                   "llmOtherProperty", "toolDef", "toolProp", "toolIdProp", 
                   "toolEndpointProp", "toolInputProperty", "toolOutputProperty", 
                   "toolDescriptionProp", "toolTypeProp", "toolOtherProperty" ]

    EOF = Token.EOF
    T__0=1
    T__1=2
    T__2=3
    T__3=4
    T__4=5
    T__5=6
    T__6=7
    T__7=8
    T__8=9
    T__9=10
    T__10=11
    T__11=12
    T__12=13
    T__13=14
    T__14=15
    T__15=16
    T__16=17
    T__17=18
    T__18=19
    T__19=20
    T__20=21
    T__21=22
    T__22=23
    T__23=24
    T__24=25
    T__25=26
    T__26=27
    T__27=28
    T__28=29
    T__29=30
    T__30=31
    T__31=32
    T__32=33
    T__33=34
    T__34=35
    TECHLAYER_RESSOURCE=36
    TECHLAYER_AIURN=37
    VAR=38
    ENTITY_ID=39
    ENTITY_VAR=40
    ENTITY_CONSTANT=41
    ENTITY_TYPE=42
    LLMPROVIDER=43
    LLMID=44
    TOOLVAR=45
    TOOLID=46
    TOOL_TYPE=47
    AGENTID=48
    AGENTNAMESPACE=49
    AGENTVAR=50
    PROPERTYVALUE=51
    THINK=52
    WS=53
    COMMENT=54
    ML_COMMENT=55

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.2")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class Ai_envDefContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def envId(self):
            return self.getTypedRuleContext(ai_environmentParser.EnvIdContext,0)


        def architectureServiceStack(self):
            return self.getTypedRuleContext(ai_environmentParser.ArchitectureServiceStackContext,0)


        def architectureAiStack(self):
            return self.getTypedRuleContext(ai_environmentParser.ArchitectureAiStackContext,0)


        def architectureDataStack(self):
            return self.getTypedRuleContext(ai_environmentParser.ArchitectureDataStackContext,0)


        def architectureAgenticMiddlewareStack(self):
            return self.getTypedRuleContext(ai_environmentParser.ArchitectureAgenticMiddlewareStackContext,0)


        def entityDef(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ai_environmentParser.EntityDefContext)
            else:
                return self.getTypedRuleContext(ai_environmentParser.EntityDefContext,i)


        def llmDef(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ai_environmentParser.LlmDefContext)
            else:
                return self.getTypedRuleContext(ai_environmentParser.LlmDefContext,i)


        def agentDef(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ai_environmentParser.AgentDefContext)
            else:
                return self.getTypedRuleContext(ai_environmentParser.AgentDefContext,i)


        def toolDef(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ai_environmentParser.ToolDefContext)
            else:
                return self.getTypedRuleContext(ai_environmentParser.ToolDefContext,i)


        def getRuleIndex(self):
            return ai_environmentParser.RULE_ai_envDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAi_envDef" ):
                listener.enterAi_envDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAi_envDef" ):
                listener.exitAi_envDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAi_envDef" ):
                return visitor.visitAi_envDef(self)
            else:
                return visitor.visitChildren(self)




    def ai_envDef(self):

        localctx = ai_environmentParser.Ai_envDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_ai_envDef)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 84
            self.match(ai_environmentParser.T__0)
            self.state = 85
            self.match(ai_environmentParser.PROPERTYVALUE)
            self.state = 86
            self.match(ai_environmentParser.T__1)
            self.state = 87
            self.match(ai_environmentParser.T__2)
            self.state = 88
            self.match(ai_environmentParser.T__1)
            self.state = 89
            self.envId()
            self.state = 90
            self.architectureServiceStack()
            self.state = 91
            self.architectureAiStack()
            self.state = 92
            self.architectureDataStack()
            self.state = 93
            self.architectureAgenticMiddlewareStack()
            self.state = 94
            self.match(ai_environmentParser.T__3)
            self.state = 95
            self.match(ai_environmentParser.T__4)
            self.state = 96
            self.match(ai_environmentParser.T__1)
            self.state = 100
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==14:
                self.state = 97
                self.entityDef()
                self.state = 102
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 103
            self.match(ai_environmentParser.T__3)
            self.state = 104
            self.match(ai_environmentParser.T__5)
            self.state = 105
            self.match(ai_environmentParser.T__1)
            self.state = 109
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==29:
                self.state = 106
                self.llmDef()
                self.state = 111
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 112
            self.match(ai_environmentParser.T__3)
            self.state = 113
            self.match(ai_environmentParser.T__6)
            self.state = 114
            self.match(ai_environmentParser.T__1)
            self.state = 118
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==19:
                self.state = 115
                self.agentDef()
                self.state = 120
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 124
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==34:
                self.state = 121
                self.toolDef()
                self.state = 126
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 127
            self.match(ai_environmentParser.T__3)
            self.state = 128
            self.match(ai_environmentParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EnvIdContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_envId

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEnvId" ):
                listener.enterEnvId(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEnvId" ):
                listener.exitEnvId(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEnvId" ):
                return visitor.visitEnvId(self)
            else:
                return visitor.visitChildren(self)




    def envId(self):

        localctx = ai_environmentParser.EnvIdContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_envId)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 130
            self.match(ai_environmentParser.T__7)
            self.state = 131
            self.match(ai_environmentParser.T__8)
            self.state = 132
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ArchitectureServiceStackContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TECHLAYER_AIURN(self):
            return self.getToken(ai_environmentParser.TECHLAYER_AIURN, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_architectureServiceStack

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterArchitectureServiceStack" ):
                listener.enterArchitectureServiceStack(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitArchitectureServiceStack" ):
                listener.exitArchitectureServiceStack(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitArchitectureServiceStack" ):
                return visitor.visitArchitectureServiceStack(self)
            else:
                return visitor.visitChildren(self)




    def architectureServiceStack(self):

        localctx = ai_environmentParser.ArchitectureServiceStackContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_architectureServiceStack)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 134
            self.match(ai_environmentParser.T__9)
            self.state = 135
            self.match(ai_environmentParser.T__8)
            self.state = 136
            self.match(ai_environmentParser.TECHLAYER_AIURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ArchitectureAiStackContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TECHLAYER_AIURN(self):
            return self.getToken(ai_environmentParser.TECHLAYER_AIURN, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_architectureAiStack

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterArchitectureAiStack" ):
                listener.enterArchitectureAiStack(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitArchitectureAiStack" ):
                listener.exitArchitectureAiStack(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitArchitectureAiStack" ):
                return visitor.visitArchitectureAiStack(self)
            else:
                return visitor.visitChildren(self)




    def architectureAiStack(self):

        localctx = ai_environmentParser.ArchitectureAiStackContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_architectureAiStack)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 138
            self.match(ai_environmentParser.T__10)
            self.state = 139
            self.match(ai_environmentParser.T__8)
            self.state = 140
            self.match(ai_environmentParser.TECHLAYER_AIURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ArchitectureDataStackContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TECHLAYER_AIURN(self):
            return self.getToken(ai_environmentParser.TECHLAYER_AIURN, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_architectureDataStack

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterArchitectureDataStack" ):
                listener.enterArchitectureDataStack(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitArchitectureDataStack" ):
                listener.exitArchitectureDataStack(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitArchitectureDataStack" ):
                return visitor.visitArchitectureDataStack(self)
            else:
                return visitor.visitChildren(self)




    def architectureDataStack(self):

        localctx = ai_environmentParser.ArchitectureDataStackContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_architectureDataStack)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 142
            self.match(ai_environmentParser.T__11)
            self.state = 143
            self.match(ai_environmentParser.T__8)
            self.state = 144
            self.match(ai_environmentParser.TECHLAYER_AIURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ArchitectureAgenticMiddlewareStackContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TECHLAYER_AIURN(self):
            return self.getToken(ai_environmentParser.TECHLAYER_AIURN, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_architectureAgenticMiddlewareStack

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterArchitectureAgenticMiddlewareStack" ):
                listener.enterArchitectureAgenticMiddlewareStack(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitArchitectureAgenticMiddlewareStack" ):
                listener.exitArchitectureAgenticMiddlewareStack(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitArchitectureAgenticMiddlewareStack" ):
                return visitor.visitArchitectureAgenticMiddlewareStack(self)
            else:
                return visitor.visitChildren(self)




    def architectureAgenticMiddlewareStack(self):

        localctx = ai_environmentParser.ArchitectureAgenticMiddlewareStackContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_architectureAgenticMiddlewareStack)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 146
            self.match(ai_environmentParser.T__12)
            self.state = 147
            self.match(ai_environmentParser.T__8)
            self.state = 148
            self.match(ai_environmentParser.TECHLAYER_AIURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EntityDefContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def entityProp(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ai_environmentParser.EntityPropContext)
            else:
                return self.getTypedRuleContext(ai_environmentParser.EntityPropContext,i)


        def getRuleIndex(self):
            return ai_environmentParser.RULE_entityDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEntityDef" ):
                listener.enterEntityDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEntityDef" ):
                listener.exitEntityDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEntityDef" ):
                return visitor.visitEntityDef(self)
            else:
                return visitor.visitChildren(self)




    def entityDef(self):

        localctx = ai_environmentParser.EntityDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_entityDef)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 150
            self.match(ai_environmentParser.T__13)
            self.state = 151
            self.match(ai_environmentParser.PROPERTYVALUE)
            self.state = 152
            self.match(ai_environmentParser.T__1)
            self.state = 156
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==15 or _la==16:
                self.state = 153
                self.entityProp()
                self.state = 158
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 159
            self.match(ai_environmentParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EntityPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def entityIdProp(self):
            return self.getTypedRuleContext(ai_environmentParser.EntityIdPropContext,0)


        def entityElementProp(self):
            return self.getTypedRuleContext(ai_environmentParser.EntityElementPropContext,0)


        def getRuleIndex(self):
            return ai_environmentParser.RULE_entityProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEntityProp" ):
                listener.enterEntityProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEntityProp" ):
                listener.exitEntityProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEntityProp" ):
                return visitor.visitEntityProp(self)
            else:
                return visitor.visitChildren(self)




    def entityProp(self):

        localctx = ai_environmentParser.EntityPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_entityProp)
        try:
            self.state = 163
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [15]:
                self.enterOuterAlt(localctx, 1)
                self.state = 161
                self.entityIdProp()
                pass
            elif token in [16]:
                self.enterOuterAlt(localctx, 2)
                self.state = 162
                self.entityElementProp()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EntityIdPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ENTITY_ID(self):
            return self.getToken(ai_environmentParser.ENTITY_ID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_entityIdProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEntityIdProp" ):
                listener.enterEntityIdProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEntityIdProp" ):
                listener.exitEntityIdProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEntityIdProp" ):
                return visitor.visitEntityIdProp(self)
            else:
                return visitor.visitChildren(self)




    def entityIdProp(self):

        localctx = ai_environmentParser.EntityIdPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_entityIdProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 165
            self.match(ai_environmentParser.T__14)
            self.state = 166
            self.match(ai_environmentParser.T__8)
            self.state = 167
            self.match(ai_environmentParser.ENTITY_ID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EntityElementPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def entityElementType(self):
            return self.getTypedRuleContext(ai_environmentParser.EntityElementTypeContext,0)


        def entityElementDescription(self):
            return self.getTypedRuleContext(ai_environmentParser.EntityElementDescriptionContext,0)


        def ENTITY_VAR(self):
            return self.getToken(ai_environmentParser.ENTITY_VAR, 0)

        def ENTITY_CONSTANT(self):
            return self.getToken(ai_environmentParser.ENTITY_CONSTANT, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_entityElementProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEntityElementProp" ):
                listener.enterEntityElementProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEntityElementProp" ):
                listener.exitEntityElementProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEntityElementProp" ):
                return visitor.visitEntityElementProp(self)
            else:
                return visitor.visitChildren(self)




    def entityElementProp(self):

        localctx = ai_environmentParser.EntityElementPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_entityElementProp)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 169
            self.match(ai_environmentParser.T__15)
            self.state = 170
            self.match(ai_environmentParser.T__8)
            self.state = 171
            _la = self._input.LA(1)
            if not(_la==40 or _la==41):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 172
            self.entityElementType()
            self.state = 173
            self.entityElementDescription()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EntityElementTypeContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ENTITY_TYPE(self):
            return self.getToken(ai_environmentParser.ENTITY_TYPE, 0)

        def ENTITY_ID(self):
            return self.getToken(ai_environmentParser.ENTITY_ID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_entityElementType

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEntityElementType" ):
                listener.enterEntityElementType(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEntityElementType" ):
                listener.exitEntityElementType(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEntityElementType" ):
                return visitor.visitEntityElementType(self)
            else:
                return visitor.visitChildren(self)




    def entityElementType(self):

        localctx = ai_environmentParser.EntityElementTypeContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_entityElementType)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 175
            self.match(ai_environmentParser.T__16)
            self.state = 176
            _la = self._input.LA(1)
            if not(_la==39 or _la==42):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class EntityElementDescriptionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_entityElementDescription

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterEntityElementDescription" ):
                listener.enterEntityElementDescription(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitEntityElementDescription" ):
                listener.exitEntityElementDescription(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitEntityElementDescription" ):
                return visitor.visitEntityElementDescription(self)
            else:
                return visitor.visitChildren(self)




    def entityElementDescription(self):

        localctx = ai_environmentParser.EntityElementDescriptionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_entityElementDescription)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 178
            self.match(ai_environmentParser.T__17)
            self.state = 179
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentDefContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def agentProperty(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ai_environmentParser.AgentPropertyContext)
            else:
                return self.getTypedRuleContext(ai_environmentParser.AgentPropertyContext,i)


        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentDef" ):
                listener.enterAgentDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentDef" ):
                listener.exitAgentDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentDef" ):
                return visitor.visitAgentDef(self)
            else:
                return visitor.visitChildren(self)




    def agentDef(self):

        localctx = ai_environmentParser.AgentDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_agentDef)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 181
            self.match(ai_environmentParser.T__18)
            self.state = 182
            self.match(ai_environmentParser.PROPERTYVALUE)
            self.state = 183
            self.match(ai_environmentParser.T__1)
            self.state = 187
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & 275413762048) != 0):
                self.state = 184
                self.agentProperty()
                self.state = 189
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 190
            self.match(ai_environmentParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def agentSystemPromptProperty(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentSystemPromptPropertyContext,0)


        def agentIdentity(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentIdentityContext,0)


        def agentNamespace(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentNamespaceContext,0)


        def agentLLMRefProperty(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentLLMRefPropertyContext,0)


        def agentToolRefProperty(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentToolRefPropertyContext,0)


        def agentInputProperty(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentInputPropertyContext,0)


        def agentOutputProperty(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentOutputPropertyContext,0)


        def agentDescriptionProp(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentDescriptionPropContext,0)


        def agentExampleProp(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentExamplePropContext,0)


        def agentTagsProp(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentTagsPropContext,0)


        def agentCustomProperty(self):
            return self.getTypedRuleContext(ai_environmentParser.AgentCustomPropertyContext,0)


        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentProperty" ):
                listener.enterAgentProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentProperty" ):
                listener.exitAgentProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentProperty" ):
                return visitor.visitAgentProperty(self)
            else:
                return visitor.visitChildren(self)




    def agentProperty(self):

        localctx = ai_environmentParser.AgentPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_agentProperty)
        try:
            self.state = 203
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [20]:
                self.enterOuterAlt(localctx, 1)
                self.state = 192
                self.agentSystemPromptProperty()
                pass
            elif token in [15]:
                self.enterOuterAlt(localctx, 2)
                self.state = 193
                self.agentIdentity()
                pass
            elif token in [21]:
                self.enterOuterAlt(localctx, 3)
                self.state = 194
                self.agentNamespace()
                pass
            elif token in [22]:
                self.enterOuterAlt(localctx, 4)
                self.state = 195
                self.agentLLMRefProperty()
                pass
            elif token in [23]:
                self.enterOuterAlt(localctx, 5)
                self.state = 196
                self.agentToolRefProperty()
                pass
            elif token in [24]:
                self.enterOuterAlt(localctx, 6)
                self.state = 197
                self.agentInputProperty()
                pass
            elif token in [25]:
                self.enterOuterAlt(localctx, 7)
                self.state = 198
                self.agentOutputProperty()
                pass
            elif token in [26]:
                self.enterOuterAlt(localctx, 8)
                self.state = 199
                self.agentDescriptionProp()
                pass
            elif token in [27]:
                self.enterOuterAlt(localctx, 9)
                self.state = 200
                self.agentExampleProp()
                pass
            elif token in [28]:
                self.enterOuterAlt(localctx, 10)
                self.state = 201
                self.agentTagsProp()
                pass
            elif token in [38]:
                self.enterOuterAlt(localctx, 11)
                self.state = 202
                self.agentCustomProperty()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentSystemPromptPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentSystemPromptProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentSystemPromptProperty" ):
                listener.enterAgentSystemPromptProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentSystemPromptProperty" ):
                listener.exitAgentSystemPromptProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentSystemPromptProperty" ):
                return visitor.visitAgentSystemPromptProperty(self)
            else:
                return visitor.visitChildren(self)




    def agentSystemPromptProperty(self):

        localctx = ai_environmentParser.AgentSystemPromptPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_agentSystemPromptProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 205
            self.match(ai_environmentParser.T__19)
            self.state = 206
            self.match(ai_environmentParser.T__8)
            self.state = 207
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentIdentityContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def AGENTID(self):
            return self.getToken(ai_environmentParser.AGENTID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentIdentity

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentIdentity" ):
                listener.enterAgentIdentity(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentIdentity" ):
                listener.exitAgentIdentity(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentIdentity" ):
                return visitor.visitAgentIdentity(self)
            else:
                return visitor.visitChildren(self)




    def agentIdentity(self):

        localctx = ai_environmentParser.AgentIdentityContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_agentIdentity)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 209
            self.match(ai_environmentParser.T__14)
            self.state = 210
            self.match(ai_environmentParser.T__8)
            self.state = 211
            self.match(ai_environmentParser.AGENTID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentNamespaceContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def AGENTNAMESPACE(self):
            return self.getToken(ai_environmentParser.AGENTNAMESPACE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentNamespace

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentNamespace" ):
                listener.enterAgentNamespace(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentNamespace" ):
                listener.exitAgentNamespace(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentNamespace" ):
                return visitor.visitAgentNamespace(self)
            else:
                return visitor.visitChildren(self)




    def agentNamespace(self):

        localctx = ai_environmentParser.AgentNamespaceContext(self, self._ctx, self.state)
        self.enterRule(localctx, 32, self.RULE_agentNamespace)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 213
            self.match(ai_environmentParser.T__20)
            self.state = 214
            self.match(ai_environmentParser.T__8)
            self.state = 215
            self.match(ai_environmentParser.AGENTNAMESPACE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentLLMRefPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LLMID(self):
            return self.getToken(ai_environmentParser.LLMID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentLLMRefProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentLLMRefProperty" ):
                listener.enterAgentLLMRefProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentLLMRefProperty" ):
                listener.exitAgentLLMRefProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentLLMRefProperty" ):
                return visitor.visitAgentLLMRefProperty(self)
            else:
                return visitor.visitChildren(self)




    def agentLLMRefProperty(self):

        localctx = ai_environmentParser.AgentLLMRefPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 34, self.RULE_agentLLMRefProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 217
            self.match(ai_environmentParser.T__21)
            self.state = 218
            self.match(ai_environmentParser.T__8)
            self.state = 219
            self.match(ai_environmentParser.LLMID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentToolRefPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TOOLID(self):
            return self.getToken(ai_environmentParser.TOOLID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentToolRefProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentToolRefProperty" ):
                listener.enterAgentToolRefProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentToolRefProperty" ):
                listener.exitAgentToolRefProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentToolRefProperty" ):
                return visitor.visitAgentToolRefProperty(self)
            else:
                return visitor.visitChildren(self)




    def agentToolRefProperty(self):

        localctx = ai_environmentParser.AgentToolRefPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 36, self.RULE_agentToolRefProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 221
            self.match(ai_environmentParser.T__22)
            self.state = 222
            self.match(ai_environmentParser.T__8)
            self.state = 223
            self.match(ai_environmentParser.TOOLID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentInputPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ENTITY_ID(self):
            return self.getToken(ai_environmentParser.ENTITY_ID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentInputProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentInputProperty" ):
                listener.enterAgentInputProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentInputProperty" ):
                listener.exitAgentInputProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentInputProperty" ):
                return visitor.visitAgentInputProperty(self)
            else:
                return visitor.visitChildren(self)




    def agentInputProperty(self):

        localctx = ai_environmentParser.AgentInputPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 38, self.RULE_agentInputProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 225
            self.match(ai_environmentParser.T__23)
            self.state = 226
            self.match(ai_environmentParser.T__8)
            self.state = 227
            self.match(ai_environmentParser.ENTITY_ID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentOutputPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ENTITY_ID(self):
            return self.getToken(ai_environmentParser.ENTITY_ID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentOutputProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentOutputProperty" ):
                listener.enterAgentOutputProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentOutputProperty" ):
                listener.exitAgentOutputProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentOutputProperty" ):
                return visitor.visitAgentOutputProperty(self)
            else:
                return visitor.visitChildren(self)




    def agentOutputProperty(self):

        localctx = ai_environmentParser.AgentOutputPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 40, self.RULE_agentOutputProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 229
            self.match(ai_environmentParser.T__24)
            self.state = 230
            self.match(ai_environmentParser.T__8)
            self.state = 231
            self.match(ai_environmentParser.ENTITY_ID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentDescriptionPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentDescriptionProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentDescriptionProp" ):
                listener.enterAgentDescriptionProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentDescriptionProp" ):
                listener.exitAgentDescriptionProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentDescriptionProp" ):
                return visitor.visitAgentDescriptionProp(self)
            else:
                return visitor.visitChildren(self)




    def agentDescriptionProp(self):

        localctx = ai_environmentParser.AgentDescriptionPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 42, self.RULE_agentDescriptionProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 233
            self.match(ai_environmentParser.T__25)
            self.state = 234
            self.match(ai_environmentParser.T__8)
            self.state = 235
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentExamplePropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentExampleProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentExampleProp" ):
                listener.enterAgentExampleProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentExampleProp" ):
                listener.exitAgentExampleProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentExampleProp" ):
                return visitor.visitAgentExampleProp(self)
            else:
                return visitor.visitChildren(self)




    def agentExampleProp(self):

        localctx = ai_environmentParser.AgentExamplePropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 44, self.RULE_agentExampleProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 237
            self.match(ai_environmentParser.T__26)
            self.state = 238
            self.match(ai_environmentParser.T__8)
            self.state = 239
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentTagsPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentTagsProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentTagsProp" ):
                listener.enterAgentTagsProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentTagsProp" ):
                listener.exitAgentTagsProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentTagsProp" ):
                return visitor.visitAgentTagsProp(self)
            else:
                return visitor.visitChildren(self)




    def agentTagsProp(self):

        localctx = ai_environmentParser.AgentTagsPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 46, self.RULE_agentTagsProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 241
            self.match(ai_environmentParser.T__27)
            self.state = 242
            self.match(ai_environmentParser.T__8)
            self.state = 243
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class AgentCustomPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def VAR(self):
            return self.getToken(ai_environmentParser.VAR, 0)

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_agentCustomProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAgentCustomProperty" ):
                listener.enterAgentCustomProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAgentCustomProperty" ):
                listener.exitAgentCustomProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitAgentCustomProperty" ):
                return visitor.visitAgentCustomProperty(self)
            else:
                return visitor.visitChildren(self)




    def agentCustomProperty(self):

        localctx = ai_environmentParser.AgentCustomPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 48, self.RULE_agentCustomProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 245
            self.match(ai_environmentParser.VAR)
            self.state = 246
            self.match(ai_environmentParser.T__8)
            self.state = 247
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LlmDefContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def llmProp(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ai_environmentParser.LlmPropContext)
            else:
                return self.getTypedRuleContext(ai_environmentParser.LlmPropContext,i)


        def getRuleIndex(self):
            return ai_environmentParser.RULE_llmDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLlmDef" ):
                listener.enterLlmDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLlmDef" ):
                listener.exitLlmDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitLlmDef" ):
                return visitor.visitLlmDef(self)
            else:
                return visitor.visitChildren(self)




    def llmDef(self):

        localctx = ai_environmentParser.LlmDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 50, self.RULE_llmDef)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 249
            self.match(ai_environmentParser.T__28)
            self.state = 250
            self.match(ai_environmentParser.PROPERTYVALUE)
            self.state = 251
            self.match(ai_environmentParser.T__1)
            self.state = 255
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & 290984067072) != 0):
                self.state = 252
                self.llmProp()
                self.state = 257
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 258
            self.match(ai_environmentParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LlmPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def llmIdProp(self):
            return self.getTypedRuleContext(ai_environmentParser.LlmIdPropContext,0)


        def llmProviderProp(self):
            return self.getTypedRuleContext(ai_environmentParser.LlmProviderPropContext,0)


        def llmModelProp(self):
            return self.getTypedRuleContext(ai_environmentParser.LlmModelPropContext,0)


        def llmEndpointProp(self):
            return self.getTypedRuleContext(ai_environmentParser.LlmEndpointPropContext,0)


        def llmVersionProp(self):
            return self.getTypedRuleContext(ai_environmentParser.LlmVersionPropContext,0)


        def llmOtherProperty(self):
            return self.getTypedRuleContext(ai_environmentParser.LlmOtherPropertyContext,0)


        def getRuleIndex(self):
            return ai_environmentParser.RULE_llmProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLlmProp" ):
                listener.enterLlmProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLlmProp" ):
                listener.exitLlmProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitLlmProp" ):
                return visitor.visitLlmProp(self)
            else:
                return visitor.visitChildren(self)




    def llmProp(self):

        localctx = ai_environmentParser.LlmPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 52, self.RULE_llmProp)
        try:
            self.state = 266
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [15]:
                self.enterOuterAlt(localctx, 1)
                self.state = 260
                self.llmIdProp()
                pass
            elif token in [30]:
                self.enterOuterAlt(localctx, 2)
                self.state = 261
                self.llmProviderProp()
                pass
            elif token in [31]:
                self.enterOuterAlt(localctx, 3)
                self.state = 262
                self.llmModelProp()
                pass
            elif token in [32]:
                self.enterOuterAlt(localctx, 4)
                self.state = 263
                self.llmEndpointProp()
                pass
            elif token in [33]:
                self.enterOuterAlt(localctx, 5)
                self.state = 264
                self.llmVersionProp()
                pass
            elif token in [38]:
                self.enterOuterAlt(localctx, 6)
                self.state = 265
                self.llmOtherProperty()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LlmIdPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LLMID(self):
            return self.getToken(ai_environmentParser.LLMID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_llmIdProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLlmIdProp" ):
                listener.enterLlmIdProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLlmIdProp" ):
                listener.exitLlmIdProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitLlmIdProp" ):
                return visitor.visitLlmIdProp(self)
            else:
                return visitor.visitChildren(self)




    def llmIdProp(self):

        localctx = ai_environmentParser.LlmIdPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 54, self.RULE_llmIdProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 268
            self.match(ai_environmentParser.T__14)
            self.state = 269
            self.match(ai_environmentParser.T__8)
            self.state = 270
            self.match(ai_environmentParser.LLMID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LlmProviderPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LLMPROVIDER(self):
            return self.getToken(ai_environmentParser.LLMPROVIDER, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_llmProviderProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLlmProviderProp" ):
                listener.enterLlmProviderProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLlmProviderProp" ):
                listener.exitLlmProviderProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitLlmProviderProp" ):
                return visitor.visitLlmProviderProp(self)
            else:
                return visitor.visitChildren(self)




    def llmProviderProp(self):

        localctx = ai_environmentParser.LlmProviderPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 56, self.RULE_llmProviderProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 272
            self.match(ai_environmentParser.T__29)
            self.state = 273
            self.match(ai_environmentParser.T__8)
            self.state = 274
            self.match(ai_environmentParser.LLMPROVIDER)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LlmModelPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_llmModelProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLlmModelProp" ):
                listener.enterLlmModelProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLlmModelProp" ):
                listener.exitLlmModelProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitLlmModelProp" ):
                return visitor.visitLlmModelProp(self)
            else:
                return visitor.visitChildren(self)




    def llmModelProp(self):

        localctx = ai_environmentParser.LlmModelPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 58, self.RULE_llmModelProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 276
            self.match(ai_environmentParser.T__30)
            self.state = 277
            self.match(ai_environmentParser.T__8)
            self.state = 278
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LlmEndpointPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_llmEndpointProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLlmEndpointProp" ):
                listener.enterLlmEndpointProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLlmEndpointProp" ):
                listener.exitLlmEndpointProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitLlmEndpointProp" ):
                return visitor.visitLlmEndpointProp(self)
            else:
                return visitor.visitChildren(self)




    def llmEndpointProp(self):

        localctx = ai_environmentParser.LlmEndpointPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 60, self.RULE_llmEndpointProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 280
            self.match(ai_environmentParser.T__31)
            self.state = 281
            self.match(ai_environmentParser.T__8)
            self.state = 282
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LlmVersionPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_llmVersionProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLlmVersionProp" ):
                listener.enterLlmVersionProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLlmVersionProp" ):
                listener.exitLlmVersionProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitLlmVersionProp" ):
                return visitor.visitLlmVersionProp(self)
            else:
                return visitor.visitChildren(self)




    def llmVersionProp(self):

        localctx = ai_environmentParser.LlmVersionPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 62, self.RULE_llmVersionProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 284
            self.match(ai_environmentParser.T__32)
            self.state = 285
            self.match(ai_environmentParser.T__8)
            self.state = 286
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class LlmOtherPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def VAR(self):
            return self.getToken(ai_environmentParser.VAR, 0)

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_llmOtherProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterLlmOtherProperty" ):
                listener.enterLlmOtherProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitLlmOtherProperty" ):
                listener.exitLlmOtherProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitLlmOtherProperty" ):
                return visitor.visitLlmOtherProperty(self)
            else:
                return visitor.visitChildren(self)




    def llmOtherProperty(self):

        localctx = ai_environmentParser.LlmOtherPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 64, self.RULE_llmOtherProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 288
            self.match(ai_environmentParser.VAR)
            self.state = 289
            self.match(ai_environmentParser.T__8)
            self.state = 290
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ToolDefContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def toolProp(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(ai_environmentParser.ToolPropContext)
            else:
                return self.getTypedRuleContext(ai_environmentParser.ToolPropContext,i)


        def getRuleIndex(self):
            return ai_environmentParser.RULE_toolDef

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterToolDef" ):
                listener.enterToolDef(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitToolDef" ):
                listener.exitToolDef(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitToolDef" ):
                return visitor.visitToolDef(self)
            else:
                return visitor.visitChildren(self)




    def toolDef(self):

        localctx = ai_environmentParser.ToolDefContext(self, self._ctx, self.state)
        self.enterRule(localctx, 66, self.RULE_toolDef)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 292
            self.match(ai_environmentParser.T__33)
            self.state = 293
            self.match(ai_environmentParser.PROPERTYVALUE)
            self.state = 294
            self.match(ai_environmentParser.T__1)
            self.state = 298
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & 313650085888) != 0):
                self.state = 295
                self.toolProp()
                self.state = 300
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 301
            self.match(ai_environmentParser.T__3)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ToolPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def toolIdProp(self):
            return self.getTypedRuleContext(ai_environmentParser.ToolIdPropContext,0)


        def toolInputProperty(self):
            return self.getTypedRuleContext(ai_environmentParser.ToolInputPropertyContext,0)


        def toolOutputProperty(self):
            return self.getTypedRuleContext(ai_environmentParser.ToolOutputPropertyContext,0)


        def toolEndpointProp(self):
            return self.getTypedRuleContext(ai_environmentParser.ToolEndpointPropContext,0)


        def toolTypeProp(self):
            return self.getTypedRuleContext(ai_environmentParser.ToolTypePropContext,0)


        def toolDescriptionProp(self):
            return self.getTypedRuleContext(ai_environmentParser.ToolDescriptionPropContext,0)


        def toolOtherProperty(self):
            return self.getTypedRuleContext(ai_environmentParser.ToolOtherPropertyContext,0)


        def getRuleIndex(self):
            return ai_environmentParser.RULE_toolProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterToolProp" ):
                listener.enterToolProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitToolProp" ):
                listener.exitToolProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitToolProp" ):
                return visitor.visitToolProp(self)
            else:
                return visitor.visitChildren(self)




    def toolProp(self):

        localctx = ai_environmentParser.ToolPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 68, self.RULE_toolProp)
        try:
            self.state = 310
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [15]:
                self.enterOuterAlt(localctx, 1)
                self.state = 303
                self.toolIdProp()
                pass
            elif token in [24]:
                self.enterOuterAlt(localctx, 2)
                self.state = 304
                self.toolInputProperty()
                pass
            elif token in [25]:
                self.enterOuterAlt(localctx, 3)
                self.state = 305
                self.toolOutputProperty()
                pass
            elif token in [32]:
                self.enterOuterAlt(localctx, 4)
                self.state = 306
                self.toolEndpointProp()
                pass
            elif token in [35]:
                self.enterOuterAlt(localctx, 5)
                self.state = 307
                self.toolTypeProp()
                pass
            elif token in [26]:
                self.enterOuterAlt(localctx, 6)
                self.state = 308
                self.toolDescriptionProp()
                pass
            elif token in [38]:
                self.enterOuterAlt(localctx, 7)
                self.state = 309
                self.toolOtherProperty()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ToolIdPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TOOLID(self):
            return self.getToken(ai_environmentParser.TOOLID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_toolIdProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterToolIdProp" ):
                listener.enterToolIdProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitToolIdProp" ):
                listener.exitToolIdProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitToolIdProp" ):
                return visitor.visitToolIdProp(self)
            else:
                return visitor.visitChildren(self)




    def toolIdProp(self):

        localctx = ai_environmentParser.ToolIdPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 70, self.RULE_toolIdProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 312
            self.match(ai_environmentParser.T__14)
            self.state = 313
            self.match(ai_environmentParser.T__8)
            self.state = 314
            self.match(ai_environmentParser.TOOLID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ToolEndpointPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_toolEndpointProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterToolEndpointProp" ):
                listener.enterToolEndpointProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitToolEndpointProp" ):
                listener.exitToolEndpointProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitToolEndpointProp" ):
                return visitor.visitToolEndpointProp(self)
            else:
                return visitor.visitChildren(self)




    def toolEndpointProp(self):

        localctx = ai_environmentParser.ToolEndpointPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 72, self.RULE_toolEndpointProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 316
            self.match(ai_environmentParser.T__31)
            self.state = 317
            self.match(ai_environmentParser.T__8)
            self.state = 318
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ToolInputPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ENTITY_ID(self):
            return self.getToken(ai_environmentParser.ENTITY_ID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_toolInputProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterToolInputProperty" ):
                listener.enterToolInputProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitToolInputProperty" ):
                listener.exitToolInputProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitToolInputProperty" ):
                return visitor.visitToolInputProperty(self)
            else:
                return visitor.visitChildren(self)




    def toolInputProperty(self):

        localctx = ai_environmentParser.ToolInputPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 74, self.RULE_toolInputProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 320
            self.match(ai_environmentParser.T__23)
            self.state = 321
            self.match(ai_environmentParser.T__8)
            self.state = 322
            self.match(ai_environmentParser.ENTITY_ID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ToolOutputPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def ENTITY_ID(self):
            return self.getToken(ai_environmentParser.ENTITY_ID, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_toolOutputProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterToolOutputProperty" ):
                listener.enterToolOutputProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitToolOutputProperty" ):
                listener.exitToolOutputProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitToolOutputProperty" ):
                return visitor.visitToolOutputProperty(self)
            else:
                return visitor.visitChildren(self)




    def toolOutputProperty(self):

        localctx = ai_environmentParser.ToolOutputPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 76, self.RULE_toolOutputProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 324
            self.match(ai_environmentParser.T__24)
            self.state = 325
            self.match(ai_environmentParser.T__8)
            self.state = 326
            self.match(ai_environmentParser.ENTITY_ID)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ToolDescriptionPropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_toolDescriptionProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterToolDescriptionProp" ):
                listener.enterToolDescriptionProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitToolDescriptionProp" ):
                listener.exitToolDescriptionProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitToolDescriptionProp" ):
                return visitor.visitToolDescriptionProp(self)
            else:
                return visitor.visitChildren(self)




    def toolDescriptionProp(self):

        localctx = ai_environmentParser.ToolDescriptionPropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 78, self.RULE_toolDescriptionProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 328
            self.match(ai_environmentParser.T__25)
            self.state = 329
            self.match(ai_environmentParser.T__8)
            self.state = 330
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ToolTypePropContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def TOOL_TYPE(self):
            return self.getToken(ai_environmentParser.TOOL_TYPE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_toolTypeProp

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterToolTypeProp" ):
                listener.enterToolTypeProp(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitToolTypeProp" ):
                listener.exitToolTypeProp(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitToolTypeProp" ):
                return visitor.visitToolTypeProp(self)
            else:
                return visitor.visitChildren(self)




    def toolTypeProp(self):

        localctx = ai_environmentParser.ToolTypePropContext(self, self._ctx, self.state)
        self.enterRule(localctx, 80, self.RULE_toolTypeProp)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 332
            self.match(ai_environmentParser.T__34)
            self.state = 333
            self.match(ai_environmentParser.T__8)
            self.state = 334
            self.match(ai_environmentParser.TOOL_TYPE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ToolOtherPropertyContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def VAR(self):
            return self.getToken(ai_environmentParser.VAR, 0)

        def PROPERTYVALUE(self):
            return self.getToken(ai_environmentParser.PROPERTYVALUE, 0)

        def getRuleIndex(self):
            return ai_environmentParser.RULE_toolOtherProperty

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterToolOtherProperty" ):
                listener.enterToolOtherProperty(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitToolOtherProperty" ):
                listener.exitToolOtherProperty(self)

        def accept(self, visitor:ParseTreeVisitor):
            if hasattr( visitor, "visitToolOtherProperty" ):
                return visitor.visitToolOtherProperty(self)
            else:
                return visitor.visitChildren(self)




    def toolOtherProperty(self):

        localctx = ai_environmentParser.ToolOtherPropertyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 82, self.RULE_toolOtherProperty)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 336
            self.match(ai_environmentParser.VAR)
            self.state = 337
            self.match(ai_environmentParser.T__8)
            self.state = 338
            self.match(ai_environmentParser.PROPERTYVALUE)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx








from agenterprise.agent_grammer.parser.ai_environmentListener import  ai_environmentListener
from agenterprise.model.data.ai_environment import Entity
from agenterprise.model.listener.AIURN import AIURN

class BaseAIEntityListener(ai_environmentListener):
    def __init__(self):
        super().__init__()
        self.entites = []
        self.current_entity = None
        self.current_element= None

    def enterEntityDef(self, ctx):
        super().enterEntityDef(ctx)
        self.current_entity = {
            "name": ctx.PROPERTYVALUE().getText(),
            "uid": None,
            "elements": []
        }
    def enterEntityElementProp(self, ctx):
        super().enterEntityElementProp(ctx)
        self.current_element = {
            "description" : "",
            "type": "",
            "ref" : "",
            "aiurn": AIURN(ctx.ENTITY_VAR().getText())
        }
    def enterEntityIdProp(self, ctx):
        super().enterEntityIdProp(ctx)
        self.current_entity["uid"]=ctx.ENTITY_ID().getText()

    def exitEntityElementProp(self, ctx):
        super().exitEntityElementProp(ctx)
       
        self.current_entity["elements"].append(self.current_element)

    def enterEntityElementDescription(self, ctx):
        super().enterEntityElementDescription(ctx)
        self.current_element['description'] = ctx.PROPERTYVALUE().getText()

    def enterEntityElementType(self, ctx):
        super().enterEntityElementType(ctx)
        if ctx.ENTITY_ID():
            self.current_element['ref'] = ctx.ENTITY_ID().getText()
        if ctx.ENTITY_TYPE():
            self.current_element['type'] = ctx.ENTITY_TYPE().getText()
    
    def exitEntityDef(self, ctx):
        super().exitEntityDef(ctx)
        entity = Entity(
            name=self.current_entity.get("name"),
            uid=AIURN(self.current_entity.get("uid")),
            elements=self.current_entity.get("elements")
        )
        self.entites.append(entity)
        self.current_entity = None

    


import os
import sys
from typing import List
import logging 
from antlr4 import FileStream, CommonTokenStream, ParseTreeWalker
from antlr4.error.ErrorListener import ErrorListener
from cookiecutter.main import cookiecutter
from cookiecutter.exceptions import OutputDirExistsException

from agenterprise.model.data.ai_environment import Agent, Entity, Tool
from agenterprise.model.data.ai_environment import LLM
from agenterprise.model.listener.nonfunctional.listener import NonFunctionalListener
from agenterprise.model.project import  Project
from agenterprise.agent_grammer.parser.ai_environmentLexer import ai_environmentLexer
from agenterprise.agent_grammer.parser.ai_environmentParser import ai_environmentParser

logging.basicConfig(level=logging.INFO, stream=sys.stdout, format='%(levelname)s: %(message)s')

logger = logging.getLogger(__name__)

class AIEnvironmentErrorListener(ErrorListener):
    def syntaxError(self, recognizer, offendingSymbol, line, column, msg, eroer):
        logger.error(f"Syntax error at line {line}, column {column}: {msg}")
        logger.error("Exiting due to syntax error.")
        sys.exit(1) 


def _scaffold_project_layer(target_dir, proj: Project):
    """
    Create initial project structure using the selected cookiecutter template.
    """
    logger.info(f"Scaffolding project layer in {target_dir} using template {proj.get_projectlayer()}")
    template_path =  proj.get_projectlayer()
    project_name = os.path.basename(os.path.abspath(target_dir))
    try:
        cookiecutter(
            template_path,
            directory="projectlayer",
            output_dir=os.path.dirname(os.path.abspath(target_dir)),
            no_input=True,
            extra_context={"project_name": project_name,
                           "project_build_id": proj.project_build_id,
                           "dsl_file": proj.get_dsl_filename()},
            overwrite_if_exists=True,
            skip_if_file_exists=True
        )
    except OutputDirExistsException:
        raise RuntimeError("The output directory exists already.")

def _scaffold_llm_layer(target_dir, proj: Project, llm:LLM, llms:List[LLM]):
    """
    Create layer for llms
    """
    logger.info(f"Scaffolding llm layer in {target_dir} using template {proj.get_llmlayer()}")

    llms = [llm.__dict__ for llm in llms]
    llms = dict(enumerate(llms))
    
    template_path = proj.get_llmlayer()
    project_name = os.path.basename(os.path.abspath(target_dir))
    try:
        cookiecutter(
            template_path,
            directory="llmlayer",
            output_dir=os.path.dirname(os.path.abspath(target_dir)),
            no_input=True,
            extra_context={"project_name": project_name,
                           "llm":llm.__dict__, 
                           "llms":llms, 
                           "project_build_id": proj.project_build_id,
                           "dsl_file": proj.get_dsl_filename()},
            overwrite_if_exists=True
        )
    except OutputDirExistsException:
        raise RuntimeError("The output directory exists already.")
   
def _scaffold_agent_layer(target_dir, proj: Project, agent:Agent):
    """
    Create initial project structure using the selected cookiecutter template.
    """
    logger.info(f"Scaffolding agent layer in {target_dir} using template {proj.get_agentlayer()}")
    template_path =  proj.get_agentlayer()
    project_name = os.path.basename(os.path.abspath(target_dir))
    try:
        cookiecutter(
            template_path,
            directory="agentlayer",
            output_dir=os.path.dirname(os.path.abspath(target_dir)),
            no_input=True,
            extra_context={"project_name": project_name, 
                           "agent":agent.__dict__, 
                           "project_build_id": proj.project_build_id,
                           "dsl_file": proj.get_dsl_filename()},
            overwrite_if_exists=True
        )
    except OutputDirExistsException:
        raise RuntimeError("The output directory exists already.")

def _scaffold_tool_layer(target_dir, proj: Project, tool:Tool):
    """
    Create initial project structure using the selected cookiecutter template.
    """
    logger.info(f"Scaffolding tool layer in {target_dir} using template {proj.get_toollayer()}")
    template_path = proj.get_toollayer()
    project_name = os.path.basename(os.path.abspath(target_dir))
    try:
        cookiecutter(
            template_path,
            directory="toollayer",
            output_dir=os.path.dirname(os.path.abspath(target_dir)),
            no_input=True,
            extra_context={"project_name": project_name, 
                           "tool":tool.__dict__, 
                           "project_build_id": proj.project_build_id,
                           "dsl_file": proj.get_dsl_filename()},
            overwrite_if_exists=True
        )
    except OutputDirExistsException:
        raise RuntimeError("The output directory exists already.")
    
def _scaffold_entity_layer(target_dir, proj: Project, entity:Entity):
    """
    Create entities.
    """
    logger.info(f"Scaffolding entity layer in {target_dir} using template {proj.get_entitylayer()}")
    template_path = proj.get_entitylayer()
    project_name = os.path.basename(os.path.abspath(target_dir))
    try:
        cookiecutter(
                template_path,
                directory="entitylayer",
                output_dir=os.path.dirname(os.path.abspath(target_dir)),
                no_input=True,
                extra_context={"project_name": project_name, 
                               "entity":entity.__dict__, 
                               "project_build_id": proj.project_build_id,
                               "dsl_file": proj.get_dsl_filename()},
                overwrite_if_exists=True
            )
    except OutputDirExistsException:
        raise RuntimeError("The output directory exists already.")
    

def _scaffold_service_layer(target_dir, proj: Project, agents:List[Agent], llms:List[LLM], tools:List[Tool], entities:List[Entity]):
    """
    Create initial project structure using the selected cookiecutter template.
    """
    logger.info(f"Scaffolding service layer in {target_dir} using template {proj.get_servicelayer()}")
    agents = [agent.__dict__ for agent in agents]
    agents = dict(enumerate(agents))

    llms = [llm.__dict__ for llm in llms]
    llms = dict(enumerate(llms))

    tools = [tool.__dict__ for tool in tools]
    tools = dict(enumerate(tools))

    entities = [entity.__dict__ for entity in entities]
    entities = dict(enumerate(entities))
    
    template_path =  proj.get_servicelayer()
    project_name = os.path.basename(os.path.abspath(target_dir))
    try:
        cookiecutter(
            template_path,
            directory="servicelayer",
            output_dir=os.path.dirname(os.path.abspath(target_dir)),
            no_input=True,
            extra_context={"project_name": project_name, 
                           "agents":agents, 
                           "tools":tools, 
                           "llms":llms, 
                           "entities":entities, 
                           "project_build_id": proj.project_build_id,
                           "dsl_file": proj.get_dsl_filename()},
            overwrite_if_exists=True
        )
    except OutputDirExistsException:
        raise RuntimeError("The output directory exists already.")
    
def _scaffold_agentic_middleware_layer(target_dir, proj: Project, agents:List[Agent], llms:List[LLM], tools:List[Tool], entities:List[Entity]):
    """
    Create initial project structure using the selected cookiecutter template.
    """
    logger.info(f"Scaffolding agentic middleware layer in {target_dir} using template {proj.get_agentic_middleware_layer()}")
    agents = [agent.__dict__ for agent in agents]
    agents = dict(enumerate(agents))

    llms = [llm.__dict__ for llm in llms]
    llms = dict(enumerate(llms))

    tools = [tool.__dict__ for tool in tools]
    tools = dict(enumerate(tools))

    entities = [entity.__dict__ for entity in entities]
    entities = dict(enumerate(entities))
    
    template_path =  proj.get_agentic_middleware_layer()
    project_name = os.path.basename(os.path.abspath(target_dir))
    try:
        cookiecutter(
            template_path,
            directory="middlewarelayer",
            output_dir=os.path.dirname(os.path.abspath(target_dir)),
            no_input=True,
            extra_context={"project_name": project_name, 
                           "agents":agents, 
                           "tools":tools, 
                           "llms":llms, 
                           "entities":entities, 
                           "project_build_id": proj.project_build_id,
                           "dsl_file": proj.get_dsl_filename()},
            overwrite_if_exists=True
        )
    except OutputDirExistsException:
        raise RuntimeError("The output directory exists already.")
        


def run_code_generation(dsl_file, target_dir):
    """
        run the code generation by dsl file
        1. Identifiy Project Template
        2. Start Code Generation
    """
    input_stream = FileStream(dsl_file)
    lexer = ai_environmentLexer(input_stream)
    stream = CommonTokenStream(lexer)
    parser = ai_environmentParser(stream)
 
    parser.addErrorListener(AIEnvironmentErrorListener())
    tree = parser.ai_envDef()
    dsl_target_file = os.path.join(target_dir, "dsl", os.path.basename(dsl_file))
    os.makedirs(os.path.dirname(dsl_target_file), exist_ok=True)
    with open(dsl_target_file, "w") as f:
        f.write(str(input_stream))

    os.makedirs(os.path.join(target_dir, ".ai"), exist_ok=True)
    llms_txt_target = os.path.join(target_dir, ".ai", os.path.basename("llms.txt"))
    with open(llms_txt_target, "w") as f:
        from agenterprise.agent_grammer.support.llms_txt import llms_txt
        f.write(llms_txt)
    logger.info(f"Wrote {llms_txt_target}")  
   
    
    # Nonfunctional Part
    nonfuncListener = NonFunctionalListener()
    walker = ParseTreeWalker()
    walker.walk(nonfuncListener, tree)

    aiEnv = nonfuncListener.environment
    project = Project(ai_techstack=aiEnv.ai_techlayer,
                      service_techstack=aiEnv.service_techlayer, 
                      data_techstack=aiEnv.data_techlayer,
                      agentic_middleware_techstack=aiEnv.agentic_middleware_techlayer,
                      target_dir=target_dir,  
                      envid=aiEnv.envid,
                      dsl_file=dsl_target_file)
    _scaffold_project_layer(target_dir, project)
    
   

    # Functional Part

    #LLM Layer
    llmListener = project.llmlistener()
    walker.walk(llmListener, tree)
    for llm in llmListener.llms:
        _scaffold_llm_layer(target_dir, project, llm, llmListener.llms)

    # Agent Layer
    agentListener = project.agentlistener()
    walker.walk(agentListener, tree)
    for agent in agentListener.agents:
        _scaffold_agent_layer(target_dir, project, agent)

    # Tool Layer
    toolListener = project.toollistener()
    walker.walk(toolListener, tree)
    for tool in toolListener.tools:
        _scaffold_tool_layer(target_dir, project, tool)
    
    # Entity Layer
    entityListener = project.entitylistener()
    walker.walk(entityListener, tree)
    for entity in entityListener.entites:
        _scaffold_entity_layer(target_dir, project, entity)

    # Agentic Middleware Layer
    agenticMiddlwareListener = project.agenticMiddlewareListener()
    walker.walk(agenticMiddlwareListener, tree)
    _scaffold_agentic_middleware_layer(target_dir, project, agentListener.agents, llmListener.llms, toolListener.tools, entityListener.entites)
 
    # Service Layer
    serviceListener = project.servicelistener()
    walker.walk(serviceListener, tree)
    _scaffold_service_layer(target_dir, project, agentListener.agents, llmListener.llms, toolListener.tools, entityListener.entites)
 
    
    logger.info(f"AI Environment generated to {target_dir}")


def run_dsl_template(dsl_file):
    with open(dsl_file, "w") as f:
        from agenterprise.agent_grammer.examples.agentmicroservice import example
        f.write(example)
    logger.info(f"DSL template generated to {dsl_file}")    


def main():
    
    import argparse
    from importlib import metadata

    parser = argparse.ArgumentParser(description="AI project generator. Visit https://www.agenterprise.ai/ for more information.")
    group = parser.add_mutually_exclusive_group()
    group.add_argument("--code-generation", action="store_true", help="Generate agent code from DSL")
    group.add_argument("--dsl-template", action="store_true", help="Generate a DSL template file at the specified path")
    parser.add_argument("--dsl", type=str, help="Path to agent DSL file (required for setup/code-generation/template)")
    parser.add_argument("--target", type=str, help="Target directory for generated code (required for setup/code-generation)")
    parser.add_argument("--version", action="store_true", help="Show the installed version of Agenterprise")
    args = parser.parse_args()

    if args.version:
        version = metadata.version("agenterprise")
        print(f"Agenterprise version: {version}")
        sys.exit(0)
    elif args.code_generation and args.dsl_template:
        raise ValueError("Only one of --code-generation or --dsl-template can be specified.")   
    elif args.code_generation:
        if not args.target:
            raise ValueError("--target is required for code-generation.")
        if not args.dsl:
            raise ValueError("--dsl is required for code-generation.")
        run_code_generation(args.dsl, args.target)
    elif args.dsl_template:
        if not args.dsl:
            raise ValueError("--dsl is required for dsl-template.")
        run_dsl_template(args.dsl)
    else:
        parser.print_help()
    

if __name__ == "__main__":
   
    main()



# Generated from agenterprise/agent_grammer/parser/ai_environment.g4 by ANTLR 4.13.2
from antlr4 import *
if "." in __name__:
    from .ai_environmentParser import ai_environmentParser
else:
    from ai_environmentParser import ai_environmentParser

# This class defines a complete generic visitor for a parse tree produced by ai_environmentParser.

class ai_environmentVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by ai_environmentParser#ai_envDef.
    def visitAi_envDef(self, ctx:ai_environmentParser.Ai_envDefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#envId.
    def visitEnvId(self, ctx:ai_environmentParser.EnvIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#architectureServiceStack.
    def visitArchitectureServiceStack(self, ctx:ai_environmentParser.ArchitectureServiceStackContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#architectureAiStack.
    def visitArchitectureAiStack(self, ctx:ai_environmentParser.ArchitectureAiStackContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#architectureDataStack.
    def visitArchitectureDataStack(self, ctx:ai_environmentParser.ArchitectureDataStackContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#architectureAgenticMiddlewareStack.
    def visitArchitectureAgenticMiddlewareStack(self, ctx:ai_environmentParser.ArchitectureAgenticMiddlewareStackContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#entityDef.
    def visitEntityDef(self, ctx:ai_environmentParser.EntityDefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#entityProp.
    def visitEntityProp(self, ctx:ai_environmentParser.EntityPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#entityIdProp.
    def visitEntityIdProp(self, ctx:ai_environmentParser.EntityIdPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#entityElementProp.
    def visitEntityElementProp(self, ctx:ai_environmentParser.EntityElementPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#entityElementType.
    def visitEntityElementType(self, ctx:ai_environmentParser.EntityElementTypeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#entityElementDescription.
    def visitEntityElementDescription(self, ctx:ai_environmentParser.EntityElementDescriptionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentDef.
    def visitAgentDef(self, ctx:ai_environmentParser.AgentDefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentProperty.
    def visitAgentProperty(self, ctx:ai_environmentParser.AgentPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentSystemPromptProperty.
    def visitAgentSystemPromptProperty(self, ctx:ai_environmentParser.AgentSystemPromptPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentIdentity.
    def visitAgentIdentity(self, ctx:ai_environmentParser.AgentIdentityContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentNamespace.
    def visitAgentNamespace(self, ctx:ai_environmentParser.AgentNamespaceContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentLLMRefProperty.
    def visitAgentLLMRefProperty(self, ctx:ai_environmentParser.AgentLLMRefPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentToolRefProperty.
    def visitAgentToolRefProperty(self, ctx:ai_environmentParser.AgentToolRefPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentInputProperty.
    def visitAgentInputProperty(self, ctx:ai_environmentParser.AgentInputPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentOutputProperty.
    def visitAgentOutputProperty(self, ctx:ai_environmentParser.AgentOutputPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentDescriptionProp.
    def visitAgentDescriptionProp(self, ctx:ai_environmentParser.AgentDescriptionPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentExampleProp.
    def visitAgentExampleProp(self, ctx:ai_environmentParser.AgentExamplePropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentTagsProp.
    def visitAgentTagsProp(self, ctx:ai_environmentParser.AgentTagsPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#agentCustomProperty.
    def visitAgentCustomProperty(self, ctx:ai_environmentParser.AgentCustomPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#llmDef.
    def visitLlmDef(self, ctx:ai_environmentParser.LlmDefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#llmProp.
    def visitLlmProp(self, ctx:ai_environmentParser.LlmPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#llmIdProp.
    def visitLlmIdProp(self, ctx:ai_environmentParser.LlmIdPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#llmProviderProp.
    def visitLlmProviderProp(self, ctx:ai_environmentParser.LlmProviderPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#llmModelProp.
    def visitLlmModelProp(self, ctx:ai_environmentParser.LlmModelPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#llmEndpointProp.
    def visitLlmEndpointProp(self, ctx:ai_environmentParser.LlmEndpointPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#llmVersionProp.
    def visitLlmVersionProp(self, ctx:ai_environmentParser.LlmVersionPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#llmOtherProperty.
    def visitLlmOtherProperty(self, ctx:ai_environmentParser.LlmOtherPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#toolDef.
    def visitToolDef(self, ctx:ai_environmentParser.ToolDefContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#toolProp.
    def visitToolProp(self, ctx:ai_environmentParser.ToolPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#toolIdProp.
    def visitToolIdProp(self, ctx:ai_environmentParser.ToolIdPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#toolEndpointProp.
    def visitToolEndpointProp(self, ctx:ai_environmentParser.ToolEndpointPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#toolInputProperty.
    def visitToolInputProperty(self, ctx:ai_environmentParser.ToolInputPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#toolOutputProperty.
    def visitToolOutputProperty(self, ctx:ai_environmentParser.ToolOutputPropertyContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#toolDescriptionProp.
    def visitToolDescriptionProp(self, ctx:ai_environmentParser.ToolDescriptionPropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#toolTypeProp.
    def visitToolTypeProp(self, ctx:ai_environmentParser.ToolTypePropContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ai_environmentParser#toolOtherProperty.
    def visitToolOtherProperty(self, ctx:ai_environmentParser.ToolOtherPropertyContext):
        return self.visitChildren(ctx)



del ai_environmentParser


from agenterprise.agent_grammer.parser.ai_environmentListener import  ai_environmentListener
from agenterprise.model.data.ai_environment import Agent, Tool
from agenterprise.model.listener.AIURN import AIURN

class BaseAIToolListener(ai_environmentListener):
    def __init__(self):
        super().__init__()
        self.tools = []
        self.current_tool = None

    def enterToolDef(self, ctx):
         self.current_tool = {
            "name": ctx.PROPERTYVALUE().getText(),
            "uid": "",
            "endpoint": "",
            "type": "",
            "description" : "",
            "properties": {},
            "input": None,
            "output": None
        }
    def enterToolIdProp(self, ctx):
        super().enterToolIdProp(ctx)
        self.current_tool["uid"] = ctx.TOOLID().getText()

    def enterToolInputProperty(self, ctx):
        super().enterToolInputProperty(ctx)
        var = ctx.ENTITY_ID().getText()
        self.current_tool["input"] = AIURN(var)

    def enterToolOutputProperty(self, ctx):
        super().enterToolOutputProperty(ctx)
        var = ctx.ENTITY_ID().getText()
        self.current_tool["output"] = AIURN(var)

    def enterToolEndpointProp(self, ctx):
        super().enterToolEndpointProp(ctx)
        self.current_tool["endpoint"] = ctx.PROPERTYVALUE().getText()

    def enterToolDescriptionProp(self, ctx):
        super().enterToolDescriptionProp(ctx)
        self.current_tool["description"] = ctx.PROPERTYVALUE().getText()

    def enterToolTypeProp(self, ctx):
        super().enterToolEndpointProp(ctx)
        self.current_tool["type"] = ctx.TOOL_TYPE().getText()
    

    def exitToolDef(self, ctx):
        super().exitToolDef(ctx)
 
        tool = Tool(
            name=self.current_tool.get("name"),
            uid=AIURN(self.current_tool.get("uid")),
            description=self.current_tool.get("description"),
            toolrefs=self.current_tool.get("toolrefs"),
            endpoint=self.current_tool.get("endpoint"),
            type = self.current_tool.get("type"),
            properties=self.current_tool.get("properties", {}),
            output=self.current_tool.get("output"),
            input=self.current_tool.get("input")
        )
        self.tools.append(tool)
        self.tool = None

   



llms_txt = """# AgentEnterprise AI Environment DSL - AI Generation Guide

## Overview
This guide helps AI models generate valid AgentMicroservice DSL (Domain Specific Language) files for the AgentEnterprise framework.

## Core Structure
Every AI Environment DSL file has this top-level structure:

```
ai_environment "Name" {
    architecture { ... }
    data { ... }
    infrastructure { ... }
    functional { ... }
}
```

## 1. ARCHITECTURE SECTION
Defines the technical stack components for your AI environment.

### Purpose
Specifies which tech layers are used for different aspects of the system.

### Structure
```
architecture {
    envid = "unique-identifier"
    service-techlayer = aiurn:techlayer:[source]:[path-or-url]
    ai-techlayer = aiurn:techlayer:[source]:[path-or-url]
    data-techlayer = aiurn:techlayer:[source]:[path-or-url]
    agentic-middleware-techlayer = aiurn:techlayer:[source]:[path-or-url]
}
```

### Tech Layer AIURN Format
- Source can be: `github` or `local`
- GitHub format: `aiurn:techlayer:github:www.github.com:organization:repository-name`
- Local format: `aiurn:techlayer:local:..:templates:component-name`

### Common Tech Layers
- Service Layer: `service-layer-fastapi-base`
- AI Layer: `ai-layer-pydanticai`
- Data Layer: `data-layer-pydantic`
- Middleware: `agentic-middleware-layer-redistream`

### Example
```
architecture {
    envid = "fb98001a0ce94c44ad091de3d2e78164"
    service-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:service-layer-fastapi-base
    ai-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:ai-layer-pydanticai
    data-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:data-layer-pydantic
    agentic-middleware-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:agentic-middleware-layer-redistream
}
```

## 2. DATA SECTION
Defines the data entities and their properties used in the system.

### Purpose
Describes all data models and types used by agents and tools.

### Entity Structure
```
entity "DisplayName" {
    uid = aiurn:entity:id:entityname
    element = aiurn:entity:var:fieldname -> TYPE # "Description"
    element = aiurn:entity:var:anotherfield -> TYPE # "Description"
}
```

### Data Types
- `TEXT`: String values
- `NUMBER`: Numeric values
- `BOOL`: Boolean true/false
- `LIST`: Array/list values
- `DICT`: Dictionary/object values
- `ANY`: Any type

### Field Reference Types
- Direct type: `TEXT`, `NUMBER`, `BOOL`, etc.
- Entity reference: `aiurn:entity:id:entityname` (reference to another entity)

### Element Structure
- `aiurn:entity:var:` = variable field
- `aiurn:entity:constant:` = constant field

### Example
```
data {
    entity "Restaurant Query" {
        uid = aiurn:entity:id:restaurantquery
        element = aiurn:entity:var:question -> TEXT # "The question about restaurants"
    }
    
    entity "Restaurant Answer" {
        uid = aiurn:entity:id:restaurantanswer
        element = aiurn:entity:var:answer -> TEXT # "The answer provided"
        element = aiurn:entity:var:restaurant -> aiurn:entity:id:restaurant # "Referenced restaurant"
    }
    
    entity "Restaurant" {
        uid = aiurn:entity:id:restaurant
        element = aiurn:entity:var:name -> TEXT # "Restaurant name"
        element = aiurn:entity:var:street -> TEXT # "Street address"
        element = aiurn:entity:var:city -> TEXT # "City location"
        element = aiurn:entity:var:rating -> NUMBER # "Star rating 1-5"
    }
}
```

## 3. INFRASTRUCTURE SECTION
Defines LLM (Language Model) configurations.

### Purpose
Configures all language models available for use by agents.

### LLM Structure
```
llm "DisplayName" {
    uid = aiurn:model:id:modelname
    provider = aiurn:model:provider:[azure|openai]
    model = "model-identifier"
    endpoint = "https://api.endpoint.com/"
    version = "api-version"
    aiurn:global:var:temperature = 0.7
    aiurn:global:var:customvar = value
}
```

### LLM Providers
- `aiurn:model:provider:azure` = Microsoft Azure OpenAI
- `aiurn:model:provider:openai` = OpenAI

### Common Configuration Variables
- `temperature`: Controls creativity (0.0 = deterministic, 1.0 = creative)
- `costid`: Cost center identifier for billing
- `max_tokens`: Maximum tokens in response
- Custom variables using `aiurn:global:var:variablename`

### Example
```
infrastructure {
    llm "My LLM" {
        uid = aiurn:model:id:geepeetee
        provider = aiurn:model:provider:azure
        model = "gpt-4o"
        endpoint = "https://any.openai.azure.com/"
        version = "2025-01-01-preview"
        aiurn:global:var:temperature = 0.7
        aiurn:global:var:costid = "ewe3949"
    }
}
```

## 4. FUNCTIONAL SECTION
Defines Agents and Tools - the core business logic.

### 4A. AGENTS
Autonomous entities that perform tasks using LLMs.

#### Agent Structure
```
agent "DisplayName" {
    uid = aiurn:agent:id:agentname
    namespace = aiurn:ns:domain:subdomain
    systemprompt = "System instruction for the agent"
    llmref = aiurn:model:id:modelname
    description = "What this agent does"
    example = "Example input"
    example = "Another example"
    tag = "Category"
    tag = "Another category"
    toolref = aiurn:tool:id:toolname
    in = aiurn:entity:id:inputentity
    out = aiurn:entity:id:outputentity
    aiurn:global:var:name = "Agent Name"
    aiurn:global:var:role = "role"
    aiurn:global:var:lifecycle = "permanent"
    aiurn:global:var:events = "event-name"
}
```

#### Required Properties
- `uid`: Unique identifier in format `aiurn:agent:id:name`
- `namespace`: Hierarchical namespace `aiurn:ns:domain:subdomain`
- `systemprompt`: The system prompt instructions
- `llmref`: Reference to LLM defined in infrastructure
- `in`: Input entity type
- `out`: Output entity type

#### Optional Properties
- `description`: What the agent does
- `example`: Example interactions (can have multiple)
- `tag`: Categorization tags (can have multiple)
- `toolref`: Tools available to agent (can have multiple)
- Custom variables via `aiurn:global:var:name`

#### Namespace Format
`aiurn:ns:mainarea:subarea`

Examples:
- `aiurn:ns:kitchen:cooking`
- `aiurn:ns:frontdesk:reservations`
- `aiurn:ns:payment:processing`

#### Example
```
agent "Cook" {
    uid = aiurn:agent:id:cook
    namespace = aiurn:ns:moewe:kitchen
    systemprompt = "You're a four star rated chef working at our restaurant"
    llmref = aiurn:model:id:geepeetee
    description = "A Cook in a restaurant"
    example = "How do I prepare pasta?"
    example = "What's the special today?"
    tag = "Restaurant"
    tag = "Recipes"
    toolref = aiurn:tool:id:crawler:v2
    in = aiurn:entity:id:restaurantquery
    out = aiurn:entity:id:restaurantanswer
    aiurn:global:var:name = "Chef Max"
    aiurn:global:var:role = "cook"
    aiurn:global:var:lifecycle = "permanent"
}
```

### 4B. TOOLS
Functions or services that agents can use to accomplish tasks.

#### Tool Structure
```
tool "DisplayName" {
    uid = aiurn:tool:id:toolname
    type = aiurn:tooltype:[mcp|openapi|code|ressource]
    endpoint = "implementation endpoint"
    description = "What this tool does"
    in = aiurn:entity:id:inputentity
    out = aiurn:entity:id:outputentity
    aiurn:global:var:customvar = value
}
```

#### Tool Types
- `aiurn:tooltype:code` = Inline Python/code implementation
- `aiurn:tooltype:mcp` = Model Context Protocol server
- `aiurn:tooltype:openapi` = OpenAPI/REST endpoint
- `aiurn:tooltype:ressource` = External resource

#### Code Tool Endpoint
For inline code tools, use lambda expressions:
```
endpoint = "lambda input: ToolOutputType(result=calculation)"
```

#### MCP Tool Endpoint
For Model Context Protocol tools:
```
endpoint = "https://mcp-server.com/endpoint"
```

#### Required Properties
- `uid`: Unique identifier in format `aiurn:tool:id:name`
- `type`: Tool type (one of the four types above)
- `endpoint`: Implementation details
- `description`: What the tool does

#### Optional Properties
- `in`: Input entity type
- `out`: Output entity type
- Custom variables via `aiurn:global:var:name`

#### Examples

**Code Tool:**
```
tool "BMI Calculator" {
    uid = aiurn:tool:id:bmi:v1
    type = aiurn:tooltype:code
    endpoint = "lambda bmiquery: ToolOutputType(bmi=round(bmiquery.weight / (bmiquery.height ** 2), 2))"
    description = "Tool calculating the BMI by weight and height"
    in = aiurn:entity:id:bmiquery
    out = aiurn:entity:id:bmiresult
}
```

**MCP Tool:**
```
tool "Web Crawler" {
    uid = aiurn:tool:id:crawler:v2
    type = aiurn:tooltype:mcp
    endpoint = "https://remote.mcpservers.org/fetch/mcp"
    description = "Tool for fetching and parsing webpages"
}
```

**OpenAPI Tool:**
```
tool "Weather Service" {
    uid = aiurn:tool:id:weather:v1
    type = aiurn:tooltype:openapi
    endpoint = "https://api.weather.service/v1"
    description = "Get weather information for any location"
}
```

## 5. AIURN IDENTIFIER FORMATS

### Entity IDs
`aiurn:entity:id:entityname`
- Used in `uid` fields of entities
- Used to reference entity types in `in` and `out` properties

### Entity Variables
`aiurn:entity:var:fieldname`
- Used to define entity fields
- Scope: within entity definition

### Entity Constants
`aiurn:entity:constant:fieldname`
- Used for constant fields
- Scope: within entity definition

### Agent IDs
`aiurn:agent:id:agentname`
- Used in `uid` fields of agents
- Used to reference agents in `llmref` properties

### Agent Namespaces
`aiurn:ns:domain:subdomain`
- Hierarchical organization
- Can have multiple levels (3+ parts)

### LLM IDs
`aiurn:model:id:modelname`
- Used in `uid` fields of LLMs
- Used in agent `llmref` properties

### Tool IDs
`aiurn:tool:id:toolname`
- Used in `uid` fields of tools
- Used in agent `toolref` properties
- Can include version: `aiurn:tool:id:toolname:v1`

### Global Variables
`aiurn:global:var:variablename`
- Used to set custom configuration
- Available on agents, tools, and LLMs
- Value types: number, string, boolean

## 6. COMPLETE EXAMPLE

```
ai_environment "Restaurant Management" {
    architecture {
        envid = "fb98001a0ce94c44ad091de3d2e78164"
        service-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:service-layer-fastapi-base
        ai-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:ai-layer-pydanticai
        data-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:data-layer-pydantic
        agentic-middleware-techlayer = aiurn:techlayer:github:www.github.com:agenterprise:agentic-middleware-layer-redistream
    }
    
    data {
        entity "User Query" {
            uid = aiurn:entity:id:userquery
            element = aiurn:entity:var:question -> TEXT # "User's question"
        }
        
        entity "Restaurant Info" {
            uid = aiurn:entity:id:restaurant
            element = aiurn:entity:var:name -> TEXT # "Restaurant name"
            element = aiurn:entity:var:rating -> NUMBER # "Star rating"
        }
    }
    
    infrastructure {
        llm "Azure GPT-4" {
            uid = aiurn:model:id:gpt4
            provider = aiurn:model:provider:azure
            model = "gpt-4o"
            endpoint = "https://api.openai.azure.com/"
            version = "2025-01-01-preview"
            aiurn:global:var:temperature = 0.7
        }
    }
    
    functional {
        agent "Restaurant Advisor" {
            uid = aiurn:agent:id:advisor
            namespace = aiurn:ns:restaurant:recommendations
            systemprompt = "You are a helpful restaurant recommendation assistant"
            llmref = aiurn:model:id:gpt4
            description = "Provides restaurant recommendations"
            example = "What's a good restaurant for Italian food?"
            tag = "Recommendations"
            in = aiurn:entity:id:userquery
            out = aiurn:entity:id:restaurant
            aiurn:global:var:name = "Restaurant Advisor"
        }
        
        tool "Restaurant Search" {
            uid = aiurn:tool:id:search:v1
            type = aiurn:tooltype:openapi
            endpoint = "https://restaurants.api.com/search"
            description = "Search for restaurants by name, cuisine, or location"
        }
    }
}
```

## 7. NAMING CONVENTIONS

### Entity Names
- Use PascalCase: `RestaurantQuery`, `UserProfile`, `OrderItem`
- Be descriptive: `RestaurantReservationRequest` not just `Request`

### Agent Names
- Use PascalCase: `CookAgent`, `WaiterAssistant`
- Descriptive role: `ReservationManager`, `BillingProcessor`

### Tool Names
- Use snake_case in IDs: `aiurn:tool:id:web_crawler:v2`
- PascalCase in display names: `"Web Crawler"`
- Include version: `v1`, `v2`

### Agent Namespaces
- Use lowercase with underscores: `aiurn:ns:kitchen:food_prep`
- Hierarchical: `aiurn:ns:domain:subdomain:component`

## 8. GENERATION RULES

When generating DSL files, follow these rules:

1. **Identifiers must be unique**: Each `uid` must be unique across the entire file
2. **References must exist**: Any `ref` or `id` reference must have a corresponding definition
3. **Required fields**: All required fields must be present
4. **Type consistency**: Referenced entity types must be defined in the data section
5. **Namespace hierarchy**: Use meaningful namespaces that reflect the domain structure
6. **Comments**: Use `#` for descriptions (optional but recommended)
7. **Indentation**: Use 4 spaces for indentation
8. **Strings**: Property values should be wrapped in double quotes

## 9. VALIDATION CHECKLIST

Before generating a DSL file, verify:

- [ ] Exactly one `ai_environment` block
- [ ] All four sections present: architecture, data, infrastructure, functional
- [ ] All referenced LLM IDs exist in infrastructure section
- [ ] All referenced entity IDs exist in data section
- [ ] All referenced tool IDs exist in functional section
- [ ] No duplicate UIDs
- [ ] All namespace hierarchies are consistent
- [ ] Tool types are valid (mcp, openapi, code, ressource)
- [ ] Entity types are valid (TEXT, NUMBER, BOOL, LIST, DICT, ANY)
- [ ] Provider types are valid (azure, openai)

## 10. COMMON PATTERNS

### Pattern: Multi-stage Agent Pipeline
```
entity "Input" { uid = aiurn:entity:id:input ... }
entity "Intermediate" { uid = aiurn:entity:id:intermediate ... }
entity "Output" { uid = aiurn:entity:id:output ... }

agent "Stage1" {
    uid = aiurn:agent:id:stage1
    in = aiurn:entity:id:input
    out = aiurn:entity:id:intermediate
}

agent "Stage2" {
    uid = aiurn:agent:id:stage2
    in = aiurn:entity:id:intermediate
    out = aiurn:entity:id:output
}
```

### Pattern: Agent with Multiple Tools
```
agent "MultiTool Agent" {
    uid = aiurn:agent:id:multitool
    toolref = aiurn:tool:id:tool1:v1
    toolref = aiurn:tool:id:tool2:v1
    toolref = aiurn:tool:id:tool3:v1
}
```

### Pattern: Custom Configuration Variables
```
agent "Configured Agent" {
    uid = aiurn:agent:id:configured
    aiurn:global:var:department = "Sales"
    aiurn:global:var:timeout = 30
    aiurn:global:var:retry_count = 3
}
```

---

This guide should enable AI models to generate syntactically correct and semantically meaningful AgentEnterprise DSL files.
"""